/**
 * @package     Jarvis
 *
 * @copyright   Copyright (C) 2009 - 2015 Omegatheme.com. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 * @website:	http://www.omegatheme.com
 *	Support Forum - http://www.omegatheme.com/forum/
 * 
 * @version	$Id: admin-fonts.js 61 2015-02-24 17:58:38Z linhnt $
 */
(function($) {
	"use strict";

	$.JarvisAdminFonts = function(parent, container) {
		this.parent = parent;
		this.container = container;
		this.addEvents();
	};

	$.JarvisAdminFonts.prototype = {
		addEvents: function() {
            
			this.container.on('click', '.jarvis-font-editor .jarvis-toolbox a[data-command]', function(e) {
				e.preventDefault();
			});

			this.container.on('click', '.jarvis-font-editor a[data-command="add"]', $.proxy(this.addItem, this));
			this.container.on('click', '.jarvis-font-editor .jarvis-toolbox a[data-command="remove"]', $.proxy(this.removeItem, this));
            
            this.container.on('change ', '.jarvis-font-editor input[type="text"]', $.proxy(this.update, this));
		},

		addItem: function(e) {
			e.preventDefault();
            var 
                item = $($('#font-item-template').html()),
                wrapper = $(e.target).closest('.jarvis-font-editor'),
                button = $(wrapper).find('a[data-command="add"]');
            item.find('input[type="text"]').attr('name', $(wrapper).attr('data-input-name'));
            item.insertBefore(button);
            
		},
        
		removeItem: function(e) {
			var
                self = this,
                item = $(e.target).closest('.jarvis-editor-item');

			if (confirm(Joomla.JText._('JARVIS_CONFIRM_REMOVE'))) {
				item.remove();
                self.update();
            }
		},
        
        update: function(e) {
            
            var 
                self = this,
                webfonts = {},
                fontfield = this.parent.wrapper.find('input.global-font-list'),
                data = this.container.serializeForm();
            if (data.jarvis) {
                if (data.jarvis.googleFonts)
                    this.parent.options.fonts.googleFonts = self.toFontObject(self.unique(
                        $.map(data.jarvis.googleFonts, function(url, i) {
                            if ($.trim(url) != '') return self.parseFontName(url, 'google');
                        })
                    ));
                
                if (data.jarvis.adobeFonts)
                    this.parent.options.fonts.adobeFonts = self.toFontObject(self.unique(
                        $.map(data.jarvis.adobeFonts, function(url, i) {
                            if ($.trim(url) != '') return self.parseFontName(url, 'adobe');
                        })
                    ));
            }
            else {
                this.parent.options.fonts.adobeFonts = [];
                this.parent.options.fonts.googleFonts = [];
            }
            $(fontfield).val(JSON.stringify(this.parent.options.fonts)).trigger('change');
        },
        
        /* 
        * return array of fonts
        */
        parseFontName: function(url, type) {
            //type = 'google|adobe'
            switch(type) {
                case 'google':
                    var query = $.url('?family', url);
                    return $.map(query.split('|'), function(f,i){
                        return f.split(':')[0];
                    });
                break;
                case 'adobe':
                    var query = $.url('filename', url).replace('.js', '');
                    return $.map(query.split(';'), function(f,i){
                        return f.split(':')[0];
                    });
                break;
            }
            
        },
        
        /* 
        * return array after remove duplicated
        */
        unique: function(arr) {
            return $.grep(arr,function(el,index){
                return index == $.inArray(el,arr);
            });
        },
        
        /* 
        * return object{'font(-/+)name': 'Font Name'} from array['font(-/+)name']
        */
        toFontObject: function(arr) {
            var obj = {};
            $.each(arr, function(i, v){
                obj[v] = v.replace(/\+/g, ' ').replace(/\-/g, ' ').ucfirst();
            });
            return obj;
        }
	};

})(jQuery);