<?php
/**
 * @package         Jarvis
 *
 * @copyright     Copyright (C) 2009 - 2015 Omegatheme.com. All rights reserved.
 * @license         GNU General Public License version 2 or later; see LICENSE
 * @website:    http://www.omegatheme.com
 *    Support Forum - http://www.omegatheme.com/forum/
 * 
 * @version $Id: Builder.php 66 2015-03-11 16:46:10Z linhnt $
 */
namespace Jarvis\Template;

defined('_JEXEC') or die('Access denied!');

class Builder
{
    protected static $instance;

    protected $document;
    protected $template;
    protected $params;
    // since 1.1
    protected $fonts = array(
        'systemFonts' => array(
            'arial' => 'Arial, Helvetica, sans-serif',
            'arial-black'   => '"Arial Black", Gadget, sans-serif',
            'comic-sans'    => '"Comic Sans MS", cursive, sans-serif',
            'impact'    => 'Impact, Charcoal, sans-serif',
            'lucida-sans'   => '"Lucida Sans Unicode", "Lucida Grande", sans-serif',
            'tahoma'    => 'Tahoma, Geneva, sans-serif',
            'trebuchet-ms'  => '"Trebuchet MS", Helvetica, sans-serif',
            'verdana'   => 'Verdana, Geneva, sans-serif',
            'courier-new'   => '"Courier New", Courier, monospace',
            'lucida-console'    => '"Lucida Console", Monaco, monospace'
        )
    );
    
    protected $systemFonts = array(
        'arial'             => 'Arial, Helvetica, sans-serif',
        'arial-black'     => '"Arial Black", Gadget, sans-serif',
        'comic-sans'        => '"Comic Sans MS", cursive, sans-serif',
        'impact'            => 'Impact, Charcoal, sans-serif',
        'lucida-sans'     => '"Lucida Sans Unicode", "Lucida Grande", sans-serif',
        'tahoma'            => 'Tahoma, Geneva, sans-serif',
        'trebuchet-ms'        => '"Trebuchet MS", Helvetica, sans-serif',
        'verdana'         => 'Verdana, Geneva, sans-serif',
        'courier-new'     => '"Courier New", Courier, monospace',
        'lucida-console'    => '"Lucida Console", Monaco, monospace'
    );

    public static function instance() {
        if (self::$instance == null) {
            self::$instance = new self();

            if (self::$instance->params == null) {
                $document = \JFactory::getDocument();
            }
        }

        return self::$instance;
    }

    private function __construct() {
        $app = \JFactory::getApplication();        
        $this->document = \JFactory::getDocument();
        $this->template = $this->document->template;
        
        $db = \JFactory::getDbo();
        //$loadedStyleId = $app->getTemplate(true)->id;
        $query = $db->getQuery(true)
            ->select('id')
            ->from('#__template_styles')
            ->where('client_id=0 AND home=1 AND template = ' . $db->quote($this->template));
                
        $db->setQuery($query);
        $defaultStyleId = $db->loadResult();
        
        $menu = $app->getMenu()->getActive();
        // Since 1.1
        // Fix Load styles
        if (is_object($menu) && $menu->template_style_id && $menu->template_style_id != $defaultStyleId) {
                $needStyleId = $menu->template_style_id;
        }
        else {
            $needStyleId = $defaultStyleId;
        }
        
        if (!empty($needStyleId)) {
            $query = $db->getQuery(true)
                    ->select('params')
                    ->from('#__jarvis_template_styles')
                    ->where('style_id = ' . $needStyleId);
                    
            $db->setQuery($query);
            $params = $db->loadResult();
            $registry = new \JRegistry;
            $registry->loadString($params);
            $this->document->params = $registry;
        }
        
        $this->params = $this->document->params;
        if ($this->params->get('fonts')) $this->fonts = array_merge($this->fonts, json_decode($this->params->get('fonts'), true));
        
    }

    public function head() {
            
        $customCode = array();
        
        if ($this->params['enableResponsive'] == 1) {
            $customCode[] = '<meta name="viewport" content="width=device-width, initial-scale=1">';
        }
        
        if (!empty($this->params['bookmarkIcon'])) {
            $extension = pathinfo($this->params['bookmarkIcon'], PATHINFO_EXTENSION);
            $href = \JURI::root(true) . '/' . $this->params['bookmarkIcon'];
            $types = array(
                'png' => 'image/png',
                'ico' => 'image/x-icon',
                'gif' => 'image/gif'
            );

            $customCode[] = "<link rel=\"Shortcut Icon\" type=\"{$types[$extension]}\" href=\"{$href}\" />";
        }

        if (!empty($this->params['xsmallIcon'])) {
            $href = \JURI::root(true) . '/' . $this->params['xsmallIcon'];
            $customCode[] = "<link rel=\"apple-touch-icon-precomposed\" sizes=\"57x57\" href=\"{$href}\" />";
        }
        if (!empty($this->params['smallIcon'])) {
            $href = \JURI::root(true) . '/' . $this->params['smallIcon'];
            $customCode[] = "<link rel=\"apple-touch-icon-precomposed\" sizes=\"72x72\" href=\"{$href}\" />";
        }
        if (!empty($this->params['mediumIcon'])) {
            $href = \JURI::root(true) . '/' . $this->params['mediumIcon'];
            $customCode[] = "<link rel=\"apple-touch-icon-precomposed\" sizes=\"114x114\" href=\"{$href}\" />";
        }
        if (!empty($this->params['largeIcon'])) {
            $href = \JURI::root(true) . '/' . $this->params['largeIcon'];
            $customCode[] = "<link rel=\"apple-touch-icon-precomposed\" sizes=\"144x144\" href=\"{$href}\" />";
        }

        $this->schemes();
        $this->typography();

        // Disable Responsive
        if ($this->params['enableResponsive'] == 0) {
            $this->document->addStyleDeclaration("
                body {
                    min-width: {$this->params['layoutWidth']}{$this->params['layoutWidthUnit']} !important;
                }
                .container {
                    width: {$this->params['layoutWidth']}{$this->params['layoutWidthUnit']} !important;
                }
            ");
        }

        // Custom css
        if (!empty($this->params['customCSS'])) {
            $this->document->addStyleDeclaration($this->params['customCSS']);
        }

        if (!empty($this->params['scriptAnalytics']) && $this->params['analyticsPosition'] == 'head') {
            $this->document->addScriptDeclaration($this->params['scriptAnalytics']);
        }
        if (!empty($this->params['customScriptHead'])) {
            $this->document->addScriptDeclaration($this->params['customScriptHead']);
        }

        $this->document->addCustomTag(
            implode("\r\n    ", $customCode)
        );

        if (($this->params['layoutStyle'] == 'boxed') || (isset($this->params['styleSwitcher']) && $this->params['styleSwitcher'] == 1)) {
            $maxWidth     = intval($this->params['layoutWidth']);
            $maxWidthUnit = $this->params['layoutWidthUnit'];

            $marginTop        = intval('0' . $this->params['boxedMarginTop']);
            $marginBottom = intval('0' . $this->params['boxedMarginBottom']);

            $this->document->addStyleDeclaration("
                .boxed .wrapper {
                    max-width: {$maxWidth}{$maxWidthUnit};
                }
                .boxed .body-bg {
                    padding-top: {$marginTop}px;
                    padding-bottom: {$marginBottom}px;
                }
            ");
                        
            if ($this->params['backgroundImage'] != '') {
                $url = \JURI::root() . $this->params['backgroundImage'];
                $this->document->addStyleDeclaration("
                    .boxed .body-bg {
                        background-image: url({$url});
                    }
                ");
            }
            else
            if ($this->params['boxedBackground'] != 'none') {
                $url = \JURI::root(true) . "/templates/{$this->template}/assets/images/patterns/{$this->params['boxedBackground']}";
                $this->document->addStyleDeclaration("
                    .boxed .body-bg {
                        background-image: url({$url});
                    }
                ");
            }
        }

        // Style Switcher
        if (isset($this->params['styleSwitcher']) && $this->params['styleSwitcher'] == 1) {
            $this->document->addScript(\JURI::root(true) . '/plugins/system/jarvis/assets/3rd/jquery-cookie/cookie.js');
            $this->document->addScript(\JURI::root(true) . '/plugins/system/jarvis/assets/3rd/jquery-populate/populate.js');

            $this->document->addStylesheet(\JURI::root(true) . '/plugins/system/jarvis/assets/css/style-switcher.css');
            $this->document->addScript(\JURI::root(true) . '/plugins/system/jarvis/assets/js/style-switcher.js');
            $this->document->addScriptDeclaration(
                sprintf('
                (function($) {
                    "use strict";

                    $(function() {
                        StyleSwitcher.init(%s);
                    });
                }).call(this, jQuery);
                ', json_encode(array(
                    'templateURI' => \JURI::root(true) . "/templates/{$this->template}",
                    'defaultColor' => $this->params['templateColor'],
                    'schemes' => \Jarvis\Template\Helper::getSchemes($this->template),
                    'patterns' => \Jarvis\Template\Helper::getPatterns($this->template),
                    'layout' => $this->params['layoutStyle'],
                    'scheme' => $this->params['templateColor']
                ))
            ));

            ob_start();
            include JARVIS_ROOT . '/lib/jarvis/tmpl/js-partials/style-switcher-template.php';

            $content = ob_get_clean();
            $this->document->addCustomTag("
                <script type=\"text/template\" id=\"style-switcher-template\">
                    {$content}
                </script>
            ");
        }

        echo '<jdoc:include type="head" />';
    }

    public function foot() {
        $code = array();

        if (!empty($this->params['scriptAnalytics']) && $this->params['analyticsPosition'] == 'body') {
            $code[] = $this->params['scriptAnalytics'];
        }
        if (!empty($this->params['customScriptBody'])) {
            $code[] = $this->params['customScriptBody'];
        }

        echo '<script type="text/javascript">', implode("\r\n    ", $code), '</script>';
    }

    public function bodyClasses() {
        $classes = array();

        if (!empty($this->params['layoutStyle']))
            $classes[] = $this->params['layoutStyle'];
        if (self::isHomepage()) $classes[] = 'homepage';
        echo implode(' ', $classes);
    }
    public function layout() {
                $layoutConfig = json_decode($this->params->get('layoutConfig'), true);
                $layoutSections = array();
                
                foreach($layoutConfig as $rowParams) {
                        $section = new LayoutSection($rowParams);
                        $layoutSections[] = $section->render();
                }
                
                echo implode("\r\n", $layoutSections);
    }

    private function schemes() {
        if (isset($this->params['templateColor'])) {
            $id = $this->params['templateColor'];
            $schemes = \Jarvis\Template\Helper::getSchemes($this->template);

            if ($this->params['styleSwitcher'] == 1 && isset($_COOKIE['_template_config'])) {
                $id = $_COOKIE['_template_config'];
            }

            if (isset($schemes[$id])) {
                $cssFile = \JURI::root(true) . "/templates/{$this->template}/assets/css/{$schemes[$id]['cssFile']}";
            }
            else {
                $default = array_shift($schemes);
                $cssFile = \JURI::root(true) . "/templates/{$this->template}/assets/css/{$default['cssFile']}";
            }

            $this->document->addStylesheet($cssFile, 'text/css', null, array('id' => 'template-scheme'));
        }
    }

    private function typography() {
        // fonts to load
        $gFontsToLoad = $this->params->get('googleFonts');
        $aFontsToLoad = $this->params->get('adobeFonts');
        $additionStyles = $this->params->get('additionStyles');
        // Load google fonts
        if (is_array($gFontsToLoad) && count($gFontsToLoad)) {
            foreach ($gFontsToLoad as $font) {
                $this->document->addStylesheet(str_replace(array('http:', 'https:'), '', $font));
            }
        }
        // Load adobe edge fonts
        if (is_array($aFontsToLoad) && count($aFontsToLoad)) {
            foreach ($aFontsToLoad as $font) {
                $this->document->addScript(str_replace(array('http:', 'https:'), '', $font));
            }
        }
        
        /**
         * General typography
         */
        if (array_key_exists($this->params['generalFont'], $this->fonts['systemFonts'])) {
            $generalFamily = $this->fonts['systemFonts'][$this->params['generalFont']];
        }
        else if (array_key_exists($this->params['generalFont'], $this->fonts['googleFonts'])){
            $generalFamily = "'" . $this->fonts['googleFonts'][$this->params['generalFont']] . "'";
        }
        else if (array_key_exists($this->params['generalFont'], $this->fonts['adobeFonts'])) {
            $generalFamily = "'" . $this->fonts['adobeFonts'][$this->params['generalFont']] . "'";
        }
        else {
            $generalFamily = "Arial, Helvetica, sans-serif";
        }
        
        $this->document->addStyleDeclaration("\tbody {
            font: {$this->params['generalSize']}{$this->params['generalSizeUnit']}/{$this->params['generalLineHeight']}{$this->params['generalLineHeightUnit']} {$generalFamily}; 
            line-height: {$this->params['generalLineHeight']}{$this->params['generalLineHeightUnit']};
            color: {$this->params['generalColor']}; }\r\n");

        // Generate css for additional styles
        if (is_array($additionStyles) && count($additionStyles))
        foreach($additionStyles as $styleStr) {
            $style = json_decode($styleStr, true);
            $fontStyles = array();
            if (isset($style['font-style'])) {
                if (in_array('bold', $style['font-style']))
                    $fontStyles[] = "font-weight: bold;";

                if (in_array('italic', $style['font-style']))
                    $fontStyles[] = "font-style: italic;";

                if (in_array('underline', $style['font-style']) || in_array('strike', $style['font-style'])) {
                    $textDecoration = "text-decoration:";

                    if (in_array('underline', $style['font-style']))
                        $textDecoration.= ' underline';

                    if (in_array('strike', $style['font-style']))
                        $textDecoration.= ' line-through';

                    $fontStyles[] = $textDecoration . ';';
                }
            }
            
            $fontStyles = implode(' ', $fontStyles);
            
            $fontFamily = "Arial, Helvetica, sans-serif";
            if (isset($style['font-family'])) {
                if (array_key_exists($style['font-family'], $this->fonts['systemFonts'])) {
                    $fontFamily = $this->fonts['systemFonts'][$style['font-family']];
                }
                else if (array_key_exists($style['font-family'], $this->fonts['googleFonts'])){
                    $fontFamily = "'" . $this->fonts['googleFonts'][$style['font-family']] . "'";
                }
                else if (array_key_exists($style['font-family'], $this->fonts['adobeFonts'])) {
                    $fontFamily = "'" . $this->fonts['adobeFonts'][$style['font-family']] . "'";
                }
            }
            
            $this->document->addStyleDeclaration("\t/* {$style['style-name']} */\r\n");
            $this->document->addStyleDeclaration("\t{$style['selector']} {
                font-family: {$fontFamily};
                font-size: {$style['font-size']}px;
                {$fontStyles}
                line-height: {$style['line-height']}px;
                color: {$style['color']} }\r\n");
        }
        $this->document->addStyleDeclaration(".homepage .hidden-home {display:none!important;}\r\n");
    }
    
    public static function isHomepage() {
        $app = \JFactory::getApplication();
        $menu = $app->getMenu();
        $lang = \JFactory::getLanguage();
        return ($menu->getActive() == $menu->getDefault($lang->getTag()));
    }
}
